function TableOfVars() {
    TableOfVars.parent.call(this);
    this.instances = [];
    this.scopes = [];
    this.node = document.createElement('table');
    this.node.className = 'variables';
    this.node.innerHTML = '<thead><tr><th>Name</th><th>Typ</th><th>Variablenwert</th></tr></thead><tbody></tbody>';
    this.nodeData = this.node.lastChild;
    this._addForm = new TableOfVarsAddForm(this);
}
TableOfVars.parent = TableOfVarsAbstract;
TableOfVars.prototype = Object.create(TableOfVars.parent.prototype);
TableOfVars.prototype.constructor = TableOfVars;

/** Used TableOfVarsAddForm instance */
TableOfVars.prototype._addForm = null;

/** DOM element beeing actual table */
TableOfVars.prototype.node = null;

/** May variables be overloaded? */
Object.defineProperty(TableOfVars.prototype, 'enableOverload', { enumerable: true, writable: true, value: false } );

/** All instances maintining a single object instance or an array */
Object.defineProperty(TableOfVars.prototype, 'instances', { enumerable: true, writable: true, value: [] } );

/** All scopes */
Object.defineProperty(TableOfVars.prototype, 'scopes', { enumerable: true, writable: true, value: [] } );

/** Available types of variables */
TableOfVars.varTypes = [];
Object.defineProperty(TableOfVars.prototype, 'varTypes', { enumerable: true, writable: false, value: TableOfVars.varTypes } );

/** Available classes */
TableOfVars.prototype._classes = [];
Object.defineProperty(TableOfVars.prototype, 'classes', {
    enumerable: true,
    get() { return this._classes.slice(); },
    set(v) {
        if (!(v instanceof Array))
            return;
        this._classes = v.slice();
        this._addForm.classNames = this._classes.map(({ className }) => className).filter(s => s);
    }
});

/** Can arrays be created? */
Object.defineProperty(TableOfVars.prototype, 'allowArrays', {
    enumerable: true,
    get() { return this._addForm.allowArrays; },
    set(v) { this._addForm.allowArrays = (v ? true : false); }
});

/**
* Get all VarTypeReference instances of given type within this part of the table
* @return Array     All found instances
*/
TableOfVars.prototype.getContainedVarsOfType = function(
    baseType,       // Class name or array items' variable type
    dimensions      // Array of given dimension if not zero
) {
    return [
        ...TableOfVars.parent.prototype.getContainedVarsOfType.call(this, baseType, dimensions),
        ...this.scopes.map(scope => scope.getContainedVarsOfType(baseType, dimensions)).reduce((prev, curr) => prev.concat(curr), []),
        ...this.instances.map(instance => instance.getContainedVarsOfType(baseType, dimensions)).reduce((prev, curr) => prev.concat(curr), [])
    ];
}

/**
* Add a new scope to this table
* @return TableOfVarsScope      Added instance
*/
TableOfVars.prototype.addScope = function(
    line,           // Line number of line starting this scope
    className       // Optional: Name of class to create (only if this is a scope to create an object)
) {
    if ((className === null) || (className === undefined)) {
        while (this.scopes.length && this.scopes[this.scopes.length - 1].isObjectCreation) {
            const topScope = this.scopes[this.scopes.length - 1];
            line = topScope.line;
            topScope.tryRemove(true);
        }
    }
    const scope = new TableOfVarsScope(this, line, className);
    this.scopes.push(scope);
    return scope;
}

/**
* Create and append a TBODY element
* @return HTMLTableBodyElement      Created TBODY element
*/
TableOfVars.prototype.addTbody = function() {
    return ScoTools.createHtml('tbody', this.node, null, undefined, this.node.lastChild);
}

/**
* Add a new variable (if possible)
* @return Boolean       Variable added?
*/
TableOfVars.prototype.addVar = function(
    name,           // Variable's name to display
    type,           // Variable's type (matching a this.varTypes[n].typeName)
    value,          // Optional: Value of variable to create
    dimensions,     // Optional: Array of given dimension if not zero
    instanceMapping // Optional: Object mapping saved memory pointer to actual pointers
) {
    if (this.scopes.length)
        return this.scopes[this.scopes.length - 1].addVar(name, type, value, dimensions, instanceMapping);
    return TableOfVarsAbstract.prototype.addVar.call(this, name, type, value, dimensions, instanceMapping);
}

/**
* Get information on all references to given address within table part
* @return Array     Information on references beeing objects like { type: "var", obj: "Actual variable" }
*/
TableOfVars.prototype.getExternalReferencesContained = function(
    address         // Canonic human readable memory location of instance in question
) {
    return [
        ...TableOfVarsAbstract.prototype.getExternalReferencesContained.call(this, address),
        ...this.instances.map(instance => instance.getExternalReferencesContained(address)).reduce((a, b) => a.concat(b), []),
        ...this.scopes.map(scope => scope.getExternalReferencesContained(address)).reduce((a, b) => a.concat(b), [])
    ];
}
