function VarType(
    table,      // Responsible TableOfVars instance
    varName     // Name of this variable
) {
    this.varName = varName;
    this.table = table;
    this.input = this._createInput(table.getDocument());
    this.createTr();
}

/**
* Add table rows to this.table
* @return undefined
*/
VarType.prototype.createTr = function() {
    this._tr = this.table.addTrVar(this.varName, this.typeName, this.input, () => this.remove());
}

/** Name of this variable */
VarType.prototype.varName = '';

/** Used number of bytes to store this variable */
VarType.countBytes = 4;

/** Responsible TableOfVars instance */
VarType.prototype.table = null;

/** Input element to edit value */
VarType.prototype.input = null;

/** Present value */
Object.defineProperty(VarType.prototype, 'value', { enumerable: true,
    get() { return this.input.value; },
    set(v) { if (this.input) this.input.value = v; }
});

/**
* Check if variable's value is given value
* @return Boolean       Is variable's value given value?
*/
VarType.prototype.equals = function(
    value       // Value to compare with
) {
    return (this.value == value);
}

/**
* Create a input for this type
* @return HTMLElement       Created input, select, …
*/
VarType.prototype._createInput = function(
    document        // HTMLDocument instance to use
) {
    return document.createElement('input');
}

/**
* Remove this variable
* @return undefined
*/
VarType.prototype.remove = function() {
    this.table.removeVar(this, [ this._tr ]);
}

/**
* Creates a „<variable name> = <variable value>“ string
* @return String    Created string
*/
VarType.prototype.toString = function() {
    return this.varName + ' = ' + this.value;
}

/**
* Convert contained data to JSON
* @return Object        JSON representation of contained data
*/
VarType.prototype.toJSON = function() {
    return {
        key: this.varName,
        type: this.typeName,
        value: this.value
    };
}
